import requests
from curl_cffi import requests as cffi

def get(url: str, params: dict = None, verify: bool = True):
    response = requests.get(url, params=params, verify=verify, timeout=10.0)
    response.raise_for_status()
    
    return response.content

def curl(url: str, impersonate: str = "chrome110"):
    response = cffi.get(url, impersonate=impersonate, timeout=10.0)
    response.raise_for_status()

    return response.content


def currency_converter(type: str, value, monitor: dict):
    """
    Convierte una cantidad de dinero de una moneda a otra utilizando los datos de un monitor específico.
    """
    price_monitor = monitor['price']

    try:
        if isinstance(value, int) or isinstance(value, float):
            if type == 'VES':
                return value / float(price_monitor)
            elif type in ['USD', 'EUR']:
                return value * float(price_monitor)
            else:
                raise ValueError(f"The type must be USD or VES not {type}")
    except TypeError as e:
        raise e


import pkg_resources

def _get_image_path(image_name: str) -> str:
    image_path = pkg_resources.resource_filename(__name__, image_name)
    
    return image_path

BANCAMIGA  = _get_image_path('bancamiga.png')
MERCANTIL  = _get_image_path('banco-mercantil.png')
PROVINCIAL = _get_image_path('banco-provincial.png')
BDV        = _get_image_path('banco-venezuela.png')
BANESCO    = _get_image_path('banesco.png')
BANPLUS    = _get_image_path('banplus.png')
BNC        = _get_image_path('bnc.png')

ENPARALELOVZLA = _get_image_path('vmo.png')
UPHOLD         = _get_image_path('veuph.png')
SKRILL         = _get_image_path('veskr.png')
BCV            = _get_image_path('ves.png')
PAYPAL         = _get_image_path('vepay.png')
AMAZON         = _get_image_path('veamz.png')
DOLARTODAY     = _get_image_path('vdt.png')
CRIPTODOLAR    = _get_image_path('vcrip.png')
BINANCE        = _get_image_path('vbin.png')


from . import network
from .pages import Monitor as Page
from .tools import get_time_zone as getdate, currency_converter
from .provider import select_monitor

import json
from colorama import Fore

version = '1.4.0'

def check_dependence_version():
    response = network.get("https://pypi.org/pypi/pydolarvenezuela/json")
    latest_version = json.loads(response)["info"]["version"]

    if version != latest_version:
        print(f"{Fore.GREEN}New version: {latest_version}.{Fore.RESET} {Fore.RED}Current version {version}.{Fore.RESET} write the following command: pip install --upgrade pyDolarVenezuela\n")

check_dependence_version()

class Monitor:
    def __init__(self, provider: Page, currency: str = 'USD') -> None:
        """
        La clase `Monitor` proporciona funcionalidades para consultar los precios de diversos monitores en Venezuela.

        Parámetros:
        - `provider`: La página de la que se accederán los datos.
        - `currency`: La moneda en la que se expresarán los precios. Puede ser `USD` o `EUR`. Por defecto es `USD`.
        """
        if not isinstance(provider, Page):
            raise TypeError("The parameter must be an object of type Monitor.")
        
        self.provider = provider
        self.currency = currency.lower()
    
    def get_value_monitors(self, monitor_code: str = None, name_property: str = None, prettify: bool = False):
        """
        El método `get_value_monitors` permite acceder a los datos extraídos de los monitores.

        Parámetros:
        - `monitor_code`: El código del monitor del cual se desea obtener información. Por defecto es `None`.
        - `name_property`: El nombre de la propiedad específica del diccionario de la información del monitor extraído que se desea obtener. Por defecto es `None`.
        - `prettify`: Si es True, muestra los precios en formato de moneda con el símbolo de Bolívares. Por defecto es `False`.
        """ 
        return select_monitor(self.provider, self.currency, monitor_code, name_property, prettify)

__all__ = ['pages', 'currency_converter', 'getdate', 'Monitor']


from .. import network
from ..tools import time

from bs4 import BeautifulSoup

def _convert_specific_format(text: str, character: str = '_') -> str:
    acentos = {'á': 'a', 'é': 'e', 'í': 'i', 'ó': 'o', 'ú': 'u'}
    for acento, sin_acento in acentos.items():
        text = text.lower().replace(acento, sin_acento).replace(' ', character)
    return text

def _get_values_monitors(soup: BeautifulSoup):
    return [value for value in soup]

class ExchangeMonitor:
    def __init__(self, url: str, currency: str) -> None:
        response = (network.curl(url + "dolar-venezuela") if currency == 'usd'
                    else network.curl(url + "dolar-venezuela/EUR"))
        self.soup = BeautifulSoup(response, "html.parser")
    
    def _load(self):
        section_dolar_venezuela = self.soup.find_all("div", "col-xs-12 col-sm-6 col-md-4 col-tabla")
        _scraping_monitors = _get_values_monitors(section_dolar_venezuela)

        self.data = {}
        for scraping_monitor in _scraping_monitors:
            result = scraping_monitor.find("div", "module-table module-table-fecha")

            name  = result.find("h6", "nombre").text
            price = str(result.find('p', "precio").text).replace(',', '.')

            if price.count('.') == 2:
                price = price.replace('.', '', 1)

            last_update = time.get_formatted_time(' '.join(str(result.find('p', "fecha").text).split(' ')[1:]).capitalize())
            symbol = str(result.find('p', "cambio-por").text)[0] if not str(result.find('p', "cambio-por").text)[0] == ' ' else ''
            color  = "red" if symbol == '▼' else "green" if symbol == '▲' else "neutral"
            percent = str(result.find('p', "cambio-por").text)[1:].strip()
            change = str(result.find('p', "cambio-num").text)

            data = {
                'title': name,
                'price': price,
                'last_update': last_update,
                'percent': percent,
                'change': change,
                'color': color,
                'symbol': symbol
            }

            self.data[_convert_specific_format(name)] = data
    
    def get_values(self, monitor_code: str, name_property: str, prettify: bool):
        self._load()

        if not monitor_code:
            return self.data
        
        try:
            monitor_data = self.data[monitor_code.lower()]
            if name_property:
                value = monitor_data[name_property]
                return f'Bs. {value}' if prettify and name_property == 'price' else value
            return monitor_data
        except KeyError:
            raise KeyError("Does not match any of the properties that were provided in the dictionary. Most information: https://github.com/fcoagz/pyDolarVenezuela")


currencies = {
    "eur": {"name": "Euro", "id": "euro"},
    "cny": {"name": "Yuan chino", "id": "yuan"},
    "try": {"name": "Lira turca", "id": "lira"},
    "rub": {"name": "Rublo ruso", "id": "rublo"},
    "usd": {"name": "Dólar estadounidense", "id": "dolar"}
}

monitors = {'binance': 'Binance', 'dolartoday': 'DolarToday',
            'yadio': 'Yadio', 'airtm': 'Airtm', 'cambiosrya': 'Cambios R&A',
            'mkambio': 'Mkambio', 'bcv': 'BCV', 'promediovip': 'EnParaleloVzlaVip',
            'prom_epv': 'EnParalelovzla'}

monitors_exchange = ["dolar-em", "monitor_dolar_venezuela", "enparalelovzla", "monitor_dolar_vzla", "petro",
"bcv", "remesas_zoom", "italcambio","bancamiga","banco_de_venezuela","banco-exterior",
"banplus","bnc","banesco","bbva_provincial","mercantil","otras_instituciones",
"binance","airtm","reserve","syklo","yadio",
"dolartoday","mkambio" ,"cambios-r&a" ,"paypal" ,"zinli" ,
"skrill" ,"amazon_gift_card"]

time_units = {
    "semana": "weeks", "semanas": "weeks",
    "día": "days", "días": "days",
    "horas": "hours", "hora": "hours",
    "minutos": "minutes", "minuto": "minutes",
    "segundos": "seconds", "segundo": "seconds"
}


from babel.dates import format_date, format_time
from datetime import datetime, timedelta
from pytz import timezone

from pyDolarVenezuela.utils import time_units

standard_time_zone = timezone('America/Caracas')

def get_time(date_string: str):
    """
    Formatear datetime a string.
    """
    datetime_obj = datetime.strptime(date_string, '%Y-%m-%d %H:%M')
    
    return datetime_obj.strftime('%d/%m/%Y, %I:%M %p')

def get_formatted_time(date_string: str):
    """
    Formatear string a strdatetime. Solo que esta formateada primeramente como (Hace una Hora).
    """
    datetime_obj = datetime.now(standard_time_zone)
    listdate = date_string.split(' ')

    if len(listdate) == 3:
        time = listdate[-1]

        if listdate[1] == "un" or listdate[1] == "una":
            hms = 1
        else:
            hms = int(listdate[1])
        
        duration = timedelta(**{time_units[time]: hms})
        exact_time = datetime_obj - duration

        if time == "día" or time == "días":
            return exact_time.strftime("%d/%m/%Y") 
            
        return exact_time.strftime("%d/%m/%Y, %I:%M %p")
        
    elif len(listdate) == 5:
        duration = timedelta(days=30)

        exact_time = datetime_obj - duration
        return exact_time.strftime("%d/%m/%Y")
        
    else:
        return None 

def get_time_standard(date_string: str):
    """
    Formatear datetime a string. Restando las horas que tuvo la ultima actualizacion del monitor \
    por la zona horaria universal.
    """
    rested_hours = timedelta(hours=4)
    time_zone_utc = datetime.fromisoformat(date_string[:-1])
    datetime_obj = time_zone_utc - rested_hours

    return datetime_obj.strftime('%d/%m/%Y, %I:%M %p')

def get_time_zone():
    """
    Obtener la hora actual de Venezuela.
    """
    datetime_obj = datetime.now(standard_time_zone)

    formatted_date = format_date(datetime_obj.date(), format='full', locale='es')
    formatted_time = format_time(datetime_obj.time(), format='h:mm:ss a', locale='es')

    return {
        "date": formatted_date,
        "time": formatted_time
    }


from pyDolarVenezuela.utils import currencies

import requests
from bs4 import BeautifulSoup

requests.packages.urllib3.disable_warnings()

def _get_rate_by_id(tag_id: str, soup: BeautifulSoup):
    return float(soup.find(id=tag_id).find("strong").text.strip().replace(',', '.'))

def _get_time(soup: BeautifulSoup):
    date = soup.find("span", "date-display-single")
    return date.text.strip().replace('  ', ' ')

class BCV:
    def __init__(self, url: str, currency: str) -> None:
        response      = requests.get(url, verify=False)
        response.raise_for_status()

        self.soup     = BeautifulSoup(response.content, 'html.parser')
        self.currency = currency

    def _load(self) -> None:
        section_tipo_de_cambio_oficial = self.soup.find("div", "view-tipo-de-cambio-oficial-del-bcv")

        self.rates = {}
        self.rates['last_update'] = _get_time(section_tipo_de_cambio_oficial)

        for code, values in currencies.items():
            self.rates[code] = {
                "title": values['name'],
                "price": round(_get_rate_by_id(values['id'], section_tipo_de_cambio_oficial), 2),
                "price_old": _get_rate_by_id(values['id'], section_tipo_de_cambio_oficial)
            }

    def get_values(self, currency_code: str, name_property: str, prettify: bool):
        self._load()

        if not currency_code:
            return self.rates
        
        try:
            monitor_data = self.rates[currency_code.lower()]
            if name_property:
                value = monitor_data[name_property]
                return f'Bs. {value}' if prettify and name_property == 'price' else value
            return monitor_data
        except KeyError:
            raise KeyError("Does not match any of the properties that were provided in the dictionary. Most information: https://github.com/fcoagz/pyDolarVenezuela")


from .bcv import BCV
from .criptodolar import CriptoDolar
from .exchangemonitor import ExchangeMonitor
from .ivenezuela import iVenezuela
from .dpedidos import Dpedidos

from ..pages import Monitor
from ..pages import BCV as B, CriptoDolar as C, Dpedidos as D, ExchangeMonitor as E, iVenezuela as I 

monitor_classes = {
    'usd': {
        B.name: BCV,
        C.name: CriptoDolar,
        E.name: ExchangeMonitor,
        D.name: Dpedidos,
        I.name: iVenezuela
    },
    'eur': {
        B.name: BCV,
        C.name: CriptoDolar,
        E.name: ExchangeMonitor
    }
}

def select_monitor(provider: Monitor,
                   currency: str,
                   monitor_code: str,
                   name_property: str,
                   prettify: bool):
    
    if currency not in ['usd', 'eur']:
        raise ValueError(f"The type must be 'usd' or 'eur' not {currency}")

    try:
        monitor_class = monitor_classes.get(currency).get(provider.name)
        if monitor_class is not None:
            return monitor_class(provider.provider, currency).get_values(monitor_code, name_property, prettify)
        else:
            raise ValueError(f"Provider not supported")
    except Exception as e:
        raise e


from pyDolarVenezuela import network
from pyDolarVenezuela.tools import time
from pyDolarVenezuela.utils import monitors

import json

class Dpedidos:
    def __init__(self, url: str, currency: str) -> None:
        response           = network.get(url + 'minmaxhistorial')
        json_response      = json.loads(response)

        self.json_response = json_response['result'][0]
        self.currency      = currency
    
    def get_values(self, monitor_code: str, name_property: str, prettify: bool):
        result = {}

        for key, title in monitors.items():
            if key in self.json_response and self.json_response[key] not in ['0', None]:
                data = {
                    'title': title,
                    'price': round(float(self.json_response[key]), 2)
                }

                if key == 'prom_epv':
                    data['last_update'] = time.get_time(f"{self.json_response['fecha_epv']} {'13' if self.json_response['hora_epv'] == '1' else self.json_response['hora_epv']}:00")
                
                result[title.lower()] = data
        result['last_update'] = time.get_time(f"{self.json_response['fecha']} {self.json_response['hora']}")

        if not monitor_code:
            return result
        
        try:
            monitor_data = result[monitor_code.lower()]
            if name_property:
                value = monitor_data[name_property]
                return f'Bs. {value}' if prettify and name_property == 'price' else value
            return monitor_data
        except KeyError:
            raise KeyError("Does not match any of the properties that were provided in the dictionary. Most information: https://github.com/fcoagz/pyDolarVenezuela")